@@ -20,6 +20,7 @@ from calliope.core.preprocess import \
     build_model_data, \
     apply_time_clustering, \
     final_timedimension_processing
+from calliope.core.preprocess.checks import check_future_deprecation_warnings
 from calliope.core.util.logging import log_time
 from calliope.core.util.dataset import split_loc_techs
 from calliope.core.util.tools import apply_to_dict
@@ -77,6 +78,7 @@ class Model(object):
             raise ValueError(
                 'Input configuration must either be a string or a dictionary.'
             )
+        check_future_deprecation_warnings(self._model_run, self._model_data)
 
         self.plot = plotting.ModelPlotMethods(self)
 
@@ -260,11 +262,11 @@ class Model(object):
         model_name = self._model_data.attrs.get('model.name', 'None')
         info_strings.append('Model name:   {}'.format(model_name))
         msize = '{locs} locations, {techs} technologies, {times} timesteps'.format(
-            locs=len(self._model_data.coords['locs'].values),
+            locs=len(self._model_data.coords.get('locs', [])),
             techs=(
-                len(self._model_data.coords['techs_non_transmission'].values) +
-                len(self._model_data.coords['techs_transmission_names'].values)
+                len(self._model_data.coords.get('techs_non_transmission', [])) +
+                len(self._model_data.coords.get('techs_transmission_names', []))
             ),
-            times=len(self._model_data.coords['timesteps'].values))
+            times=len(self._model_data.coords.get('timesteps', [])))
         info_strings.append('Model size:   {}'.format(msize))
         return '\n'.join(info_strings)
