@@ -58,6 +58,34 @@ class TestModelRun:
         with pytest.raises(exceptions.ModelError):
             build_model(override_dict=override, override_groups='simple_supply,one_day')
 
+    def test_conversion_plus_primary_carriers(self):
+        """
+        Test that user has input input/output primary carriers for conversion_plus techs
+        """
+        override1 = {'techs.test_conversion_plus.essentials.carrier_in': ['gas', 'coal']}
+        override2 = {'techs.test_conversion_plus.essentials.primary_carrier_in': 'coal'}
+        override3 = {'techs.test_conversion_plus.essentials.primary_carrier_out': 'coal'}
+
+        model = build_model({}, override_groups='simple_conversion_plus,two_hours')
+        assert model._model_run.techs.test_conversion_plus.essentials.get_key(
+            'primary_carrier_in', None
+        ) == 'gas'
+
+        # should fail: multiple carriers in, but no primary_carrier_in assigned
+        with pytest.raises(exceptions.ModelError) as error:
+            build_model(override1, override_groups='simple_conversion_plus,two_hours')
+        assert check_error_or_warning(error, 'Primary_carrier_in must be assigned')
+
+        # should fail: primary_carrier_in not one of the carriers_in
+        with pytest.raises(exceptions.ModelError) as error:
+            build_model(override2, override_groups='simple_conversion_plus,two_hours')
+        assert check_error_or_warning(error, 'Primary_carrier_in `coal` not one')
+
+        # should fail: primary_carrier_out not one of the carriers_out
+        with pytest.raises(exceptions.ModelError) as error:
+            build_model(override3, override_groups='simple_conversion_plus,two_hours')
+        assert check_error_or_warning(error, 'Primary_carrier_out `coal` not one')
+
     def test_incorrect_subset_time(self):
         """
         If subset_time is a list, it must have two entries (start_time, end_time)
@@ -771,6 +799,29 @@ class TestChecks:
             "Updated from coordinate system"
         )
 
+    def test_clustering_and_cyclic_storage(self):
+        """
+        Don't allow time clustering with cyclic storage if not also using
+        storage_inter_cluster
+        """
+
+        override = {
+            'model.subset_time': ['2005-01-01', '2005-01-04'],
+            'model.time': {
+                'function': 'apply_clustering',
+                'function_options': {
+                    'clustering_func': 'file=cluster_days.csv:0', 'how': 'mean',
+                    'storage_inter_cluster': False
+                }
+            },
+            'run.cyclic_storage': True
+        }
+
+        with pytest.raises(exceptions.ModelError) as error:
+            build_model(override, override_groups='simple_supply')
+
+        assert check_error_or_warning(error, 'cannot have cyclic storage')
+
 
 class TestDataset:
 
@@ -889,6 +940,68 @@ class TestDataset:
 
         assert model.inputs.timestep_resolution.to_pandas().unique() == [0.25]
 
+    def test_clustering(self):
+        """
+        On clustering, there are a few new dimensions in the model_data, and a
+        few new lookup arrays
+        """
+        override = {
+            'model.subset_time': ['2005-01-01', '2005-01-04'],
+            'model.time': {
+                'function': 'apply_clustering',
+                'function_options': {
+                    'clustering_func': 'file=cluster_days.csv:0', 'how': 'mean'
+                }
+            }
+        }
+
+        model = build_model(override, override_groups='simple_supply')
+
+        assert 'clusters' in model._model_data.dims
+        assert 'lookup_cluster_first_timestep' in model._model_data.data_vars
+        assert 'lookup_cluster_last_timestep' in model._model_data.data_vars
+        assert 'lookup_datestep_last_cluster_timestep' in model._model_data.data_vars
+        assert 'lookup_datestep_cluster' in model._model_data.data_vars
+        assert 'timestep_cluster' in model._model_data.data_vars
+
+        datesteps = model.inputs.datesteps.to_index().strftime('%Y-%m-%d')
+        daterange = pd.date_range('2005-01-01', '2005-01-04', freq='1D').strftime('%Y-%m-%d')
+        assert np.array_equal(datesteps, daterange)
+
+    def test_clustering_no_datestep(self):
+        """
+        On clustering, there are a few new dimensions in the model_data, and a
+        few new lookup arrays
+        """
+        override = {
+            'model.subset_time': ['2005-01-01', '2005-01-04'],
+            'model.time': {
+                'function': 'apply_clustering',
+                'function_options': {
+                    'clustering_func': 'file=cluster_days.csv:0', 'how': 'mean',
+                    'storage_inter_cluster': False
+                }
+            }
+        }
+
+        model = build_model(override, override_groups='simple_supply')
+
+        assert 'clusters' in model._model_data.dims
+        assert 'datesteps' not in model._model_data.dims
+        assert 'lookup_cluster_first_timestep' in model._model_data.data_vars
+        assert 'lookup_cluster_last_timestep' in model._model_data.data_vars
+        assert 'lookup_datestep_last_cluster_timestep' not in model._model_data.data_vars
+        assert 'lookup_datestep_cluster' not in model._model_data.data_vars
+        assert 'timestep_cluster' in model._model_data.data_vars
+
+    def test_future_warning(self):
+        """
+        Test and warning to be removed in v0.6.3-dev
+        """
+        with pytest.warns(FutureWarning) as warning:
+            build_model({}, override_groups='simple_storage')
+        assert check_error_or_warning(warning, 'From v0.6.3, cyclic storage will default to True.')
+
 
 class TestUtil():
     def test_concat_iterable_ensures_same_length_iterables(self):
